@@ -11,6 +11,9 @@
 *   05/10/04   DGP   Added DEV_NULL support.
 *   07/01/04   DGP   Added /10A cpu, model = 11.
 *   04/01/05   DGP   Changed signal handling.
+*   04/12/05   DGP   Changed CONOUT to use prtview in panel mode and
+*                    view mode to support multi screens.
+*   04/20/05   DGP   Ensure args don't exceed argc.
 *
 ***********************************************************************/
 
@@ -44,35 +47,43 @@ int runled = FALSE;		/* CPU running LED */
 int idleled = FALSE;		/* CPU idle LED */
 int faultled = FALSE;		/* CPU fault LED */
 int enablepanel = FALSE;	/* Enable panel */
-int devcnt = 0;			/* Device attached count */
-int model = 4;			/* Which model */
 int breakflag = FALSE;		/* No breakpoint enabled */
 int traceenable = FALSE;	/* No trace enabled */
 int traceit = FALSE;		/* No trace enabled */
 int mapenabled = FALSE;		/* Memory mapping enabled */
 int intsenabled = TRUE;		/* Interrupts enabled */
+int trclatch = FALSE;		/* Error Trace latch */
+int errindex = 0;		/* Error Trace Memory index */
+int trcindex = 0;		/* Error Trace Memory locked index */
+int devcnt = 0;			/* Attached device count */
+int model = 4;			/* Which model */
 int deferint = 0;		/* Defer interrupts inst count */
 int tracelim[2];		/* Trace memory limits */
 int tracestart = 0;		/* Trace instruction count start */
 int traceend = 0;		/* Trace instruction count end */
+int windowlen = 25;		/* Window length */
+int viewlen;			/* View length */
+int prtviewlen;			/* prtview length */
+int prtviewcol;			/* prtview column */
+
 uint16 mapcrudata = 0;		/* Memory map file CRU >1FA0 */
 uint16 curmapindex = 0;		/* Current map index */
 uint16 errcrudata = 0;		/* ERR CRU >1FC0 data */
 uint16 curinst = 0;		/* Current instruction */
+
 uint32 maplatch = 0;		/* Memory map latch */
 uint32 memlen = 56*1024;	/* Length of system memory */
 long delayclock = 0;		/* Delay clock interval */
+
 char romfile[256];		/* ROM file name */
 char view[MAXVIEW][81];		/* Messages panel */
+char prtview[MAXVIEW][81];	/* Print messages panel */
 uint8 memory[MEMSIZE];		/* The CPU memory */
 Device devices[MAXDEVICES];	/* The attached devices */
 MapFile mapfile[MAPSIZE];	/* Memory Mapping Files */
 FILE *tracefd = NULL;		/* Trace file fd */
 uint32 tracemem[16];		/* Error Trace Memory locked */
 uint32 errtrace[16];		/* Error Trace Memory /12 only */
-int errindex = 0;		/* Error Trace Memory index */
-int trcindex = 0;		/* Error Trace Memory locked index */
-int trclatch = FALSE;		/* Error Trace latch */
 
 
 static int dispaddr = 0;	/* Display area address */
@@ -102,19 +113,19 @@ sigintr (int sig)
 static void
 puthelp (void)
 {
-   screenposition (14,1);
+   screenposition (OUTPUTLINE,1);
    clearline ();
    printf ("  a - attach  b - boot    c - clrBrk  d - display g - go");
-   screenposition (15,1);
+   screenposition (OUTPUTLINE+1,1);
    clearline ();
    printf ("  h - help    k - break   l - load    m - modify  n - next");
-   screenposition (16,1);
+   screenposition (OUTPUTLINE+2,1);
    clearline ();
    printf ("  p - set pc  q - quit    r - reset   s - step    w - set wp");
-   screenposition (17,1);
+   screenposition (OUTPUTLINE+3,1);
    clearline ();
    printf ("  z - panel");
-   screenposition (18,1);
+   screenposition (OUTPUTLINE+4,1);
    clearline ();
    printf ("  ");
 }
@@ -126,21 +137,45 @@ puthelp (void)
 static void
 putview (int interactive)
 {
-   int i;
+   int i, j;
 
-   for (i = 0; i < MAXVIEW; i++)
-   {
-      if (interactive)
+   if (interactive && enablepanel)
+      for (j = 0; j < viewlen; j++)
       {
-	 if (!enablepanel && view[i][0] == '\0') return;
-	 screenposition (14+i,1);
+	 screenposition (OUTPUTLINE+j,1);
+	 clearline ();
+      }
+
+   for (j = i = 0; i < MAXVIEW; i++)
+   {
+      if (interactive && enablepanel)
+      {
+	 j++;
+	 if (j >= viewlen)
+	 {
+	    int ch;
+	    screenposition (windowlen-2, 1);
+	    clearline();
+	    printf ("press enter to continue:");
+	    ch = fgetc (stdin);
+	    if (ch == 'q') return;
+	    for (j = 0; j < viewlen; j++)
+	    {
+	       screenposition (OUTPUTLINE+j,1);
+	       clearline ();
+	    }
+	    j = 0;
+	 }
+
+	 screenposition (OUTPUTLINE+j,1);
 	 clearline ();
       }
       if (view[i][0])
       {
 	 printf ("%s", view[i]);
-	 if (!interactive) printf ("\n");
+	 if (!enablepanel) printf ("\n");
       }
+      else return;
       view[i][0] = '\0';
    }
 }
@@ -155,7 +190,7 @@ getnum (char *bp, int *val)
    char *cp;
    int i = 0;
 
-   while (isspace (*bp)) bp++;
+   while (*bp && isspace (*bp)) bp++;
    cp = bp;
    if (*bp == HEXSYM)
    {
@@ -183,16 +218,16 @@ panel (void)
    if (!isprint (c1)) c1 = '.';
    if (!isprint (c2)) c2 = '.';
 
-   screenposition (1,1);
+   screenposition (TOPLINE,1);
    clearline ();
    printf ("%s %s      /%s CPU %dK",
 	 INTRO, VERSION, models[model-4], memlen/1024);
 
-   screenposition (3,1);
+   screenposition (REGISTERLINE,1);
    printf (" PC  %04X  WP  %04X  ST  %04X  FP  %04X",
 	    pcreg, wpreg, statreg, lights);
 
-   screenposition (5,1);
+   screenposition (DISPLINE,1);
    if (model < 10)
    {
       printf (" DISP  %04X %04X %c%c    %-5.5s %-5.5s %-5.5s",
@@ -201,7 +236,7 @@ panel (void)
 	       idleled ? "IDLE" : " ",
 	       faultled ? "FAULT" : " ");
 
-      screenposition (6,1);
+      screenposition (BREAKLINE,1);
 #ifdef SHOWOLDPC
       printf (" BREAK %04X  OPC %04X    COUNT %ld",
 	       breakaddr,
@@ -221,7 +256,7 @@ panel (void)
 	       idleled ? "IDLE" : " ",
 	       faultled ? "FAULT" : " ");
 
-      screenposition (6,1);
+      screenposition (BREAKLINE,1);
 #ifdef SHOWOLDPC
       printf (" BREAK %06X  OPC %04X  COUNT %ld",
 	       breakaddr,
@@ -234,17 +269,17 @@ panel (void)
 #endif
    }
 
-   screenposition (8,1);
+   screenposition (WORKSPACELINE,1);
    printf ("Workspace:");
-   screenposition (9,1);
+   screenposition (WORKSPACELINE+1,1);
    printf (" R00 %04X  R01 %04X  R02 %04X  R03 %04X", R0, R1, R2, R3);
-   screenposition (10,1);
+   screenposition (WORKSPACELINE+2,1);
    printf (" R04 %04X  R05 %04X  R06 %04X  R07 %04X", R4, R5, R6, R7);
-   screenposition (11,1);
+   screenposition (WORKSPACELINE+3,1);
    printf (" R08 %04X  R09 %04X  R10 %04X  R11 %04X", R8, R9, R10, R11);
-   screenposition (12,1);
+   screenposition (WORKSPACELINE+4,1);
    printf (" R12 %04X  R13 %04X  R14 %04X  R15 %04X", R12, R13, R14, R15);
-   screenposition (14,1);
+   screenposition (OUTPUTLINE+prtviewlen-1,prtviewcol);
    fflush (stdout);
    pancount = FALSE;
 
@@ -306,7 +341,7 @@ clearall ()
    }
 
    lp = ROMSTART;
-   if (model > 4)
+   if (model == 5 || model > 9) 
       lp += TPCSSTART;
 
    for (i = 0; i < 1024; i+=2)
@@ -359,6 +394,16 @@ clearall ()
       intsenabled = TRUE;
    }
 
+   /*
+   ** Clear out display panels.
+   */
+
+   for (i = 0; i < MAXVIEW; i++)
+   {
+      prtview[i][0] = '\0';
+      view[i][0] = '\0';
+   }
+
    return (0);
 }
 
@@ -458,6 +503,8 @@ docommand (int interactive)
       break;
 
    case 'g': /* Go - run program */
+      if (enablepanel)
+         prtpanel (prtviewcol, TRUE);
       action = RUN_ACTION;
       run = TRUE;
       /*signal (SIGINT, sigintr);*/
@@ -582,6 +629,8 @@ docommand (int interactive)
 	 clearscreen ();
 	 panel ();
       }
+      else
+         screenposition (windowlen,1);
       break;
 
    default: puthelp ();
@@ -628,6 +677,7 @@ main (int argc, char **argv)
 {
    char *bp;
    char *pp;
+   char *objfile = NULL;
    char *config = NULL;
    int action;
    int i;
@@ -646,16 +696,19 @@ main (int argc, char **argv)
          {
 	 case 'C': /* delay Clock interval */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    delayclock = atoi (argv[i]);
 	    break;
 
 	 case 'c': /* Config file spec */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    config = argv[i];
 	    break;
 
 	 case 'm': /* CPU model */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    model = 0;
 	    for (pp = argv[i]; *pp; pp++)
 	    {
@@ -695,11 +748,13 @@ main (int argc, char **argv)
 
 	 case 'r': /* ROM file spec */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    strcpy (romfile, argv[i]);
 	    break;
 
 	 case 's': /* Size of memory */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    memlen = 0;
 	    for (pp = argv[i]; *pp; pp++)
 	    {
@@ -728,13 +783,31 @@ main (int argc, char **argv)
 	    }
 	    break;
 
+	 case 'w': /* Set window length */
+	    i++;
+	    if (i >= argc) goto USAGE;
+	    windowlen = atoi(argv[i]);
+	    break;
+
          default:
       USAGE:
-	    printf (
-	 "usage:  sim990 [-c config][-m model][-p][-r romfile][-s memsize]\n");
+	    printf ("usage:  sim990 [-options][object.file]\n");
+	    printf (" options:\n");
+	    printf ("   -c configfile  - Config file\n");
+	    printf ("   -m model       - CPU model\n");
+	    printf ("            CPU = 4, 5, 9, 10, 10A, 12, default = 4\n");
+	    printf ("   -p             - Enable panel\n");
+	    printf ("   -r romfile     - ROM file\n");
+	    printf ("   -s memsize     - Memory size\n");
+	    printf ("   -w NN          - Window size for panel\n");
 	    return (ABORT);
          }
       }
+      else
+      {
+	 if (objfile) goto USAGE;
+	 objfile = argv[i];
+      }
    }
 
    /*
@@ -784,9 +857,14 @@ main (int argc, char **argv)
    */
 
    clearall ();
+
    tracelim[0] = 0;
    tracelim[1] = 0xFFFE;
 
+   viewlen = windowlen - 17;
+   prtviewlen = windowlen - 15;
+   prtviewcol = 1;
+
    /*
    ** Set up initial devices
    */
@@ -841,6 +919,21 @@ main (int argc, char **argv)
       readconfig (config);
    }
 
+   /*
+   ** If object file specified, load and run it.
+   */
+
+   if (objfile)
+   {
+      if (binloader (objfile, 0) == 0)
+      {
+	 if (run)
+	 {
+	    runprog ();
+	 }
+      }
+   }
+
    /*
    ** Command loop
    */
@@ -856,10 +949,10 @@ main (int argc, char **argv)
 
       if (enablepanel)
       {
-	 screenposition (23,1);
+	 screenposition (windowlen-2,1);
 	 clearline ();
 	 printf (":%s", oinput);
-	 screenposition (23,2);
+	 screenposition (windowlen-2,2);
       }
       else
          printf ("\n:");
@@ -899,12 +992,12 @@ main (int argc, char **argv)
 	 }
 	 else
 	 {
-	    CLOSE ((int)devices[i].fd);
+	    CLOSE (devices[i].info.vdtinfo.sfd);
 	 }
       }
    }
 
    ttclose ();
-   screenposition (25,1);
+   screenposition (windowlen,1);
    return (NORMAL);
 }
