@@ -796,17 +796,16 @@ dskboot (Device *dev)
    lp = LOADADDRESS;
    SET_MASK (0);
 
+   if (model == 4 || model == 9)
+   {
+      sprintf (view[0], "dskboot: TILINE not supported on 990/%d\n", model);
+      return (-1);
+   }
+
 #ifdef USEDSKROM
-   if (model < 5)
-   {
-      wpreg = GETMEM0 (0xFFFC) & 0xFFFE;
-      pcreg = GETMEM0 (0xFFFE) & 0xFFFE;
-   }
-   else
-   {
-      wpreg = GETMEM0 (TPCSSTART+0xFFFC) & 0xFFFE;
-      pcreg = GETMEM0 (TPCSSTART+0xFFFE) & 0xFFFE;
-   }
+   wpreg = GETMEM0 (TPCSSTART+0xFFFC) & 0xFFFE;
+   pcreg = GETMEM0 (TPCSSTART+0xFFFE) & 0xFFFE;
+
    PUTREG (0, 0xFFFF);
    PUTREG (1, dev->devaddr);
    PUTREG (2, dev->unit << 8);
