@@ -140,7 +140,7 @@ getlong (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = ((uint32)GETMEM (sa & 0xFFFE, srcdst) << 16) | 
 	      (uint32)GETMEM ((sa+2) & 0xFFFE, srcdst);
       break;
@@ -200,7 +200,7 @@ getstack (uint16 sr, uint16 st, int inc, int srcdst, uint16 *stk)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sa &= 0xFFFE;
       stk[0] = GETMEM (sa,     srcdst);
       stk[1] = GETMEM (sa + 2, srcdst);
@@ -261,7 +261,7 @@ gettbl (uint16 sr, uint16 st, int inc, int srcdst, uint16 *tbl)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sa &= 0xFFFE;
       tbl[0] = GETMEM (sa,     srcdst);
       tbl[1] = GETMEM (sa + 2, srcdst);
@@ -326,7 +326,7 @@ getlscb (uint16 sr, uint16 st, int inc, int srcdst, uint16 *lscb)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sa &= 0xFFFE;
       lscb[0] = GETMEM (sa,     srcdst);
       lscb[1] = GETMEM (sa + 2, srcdst);
@@ -393,7 +393,7 @@ getquad (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = ((t_uint64)GETMEM (sa & 0xFFFE, srcdst) << 48) | 
 	     ((t_uint64)GETMEM ((sa+2) & 0xFFFE, srcdst) << 32) |
 	     ((t_uint64)GETMEM ((sa+4) & 0xFFFE, srcdst) << 16) |
@@ -454,7 +454,7 @@ putlong (uint16 dr, uint16 dt, uint32 dval, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEM (da, (uint16)((dval >> 16) & 0xFFFF), srcdst);
       PUTMEM (da+2, (uint16)(dval & 0xFFFF), srcdst);
       break;
@@ -511,7 +511,7 @@ putstack (uint16 dr, uint16 dt, uint16 *stk, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       da &= 0xFFFE;
       PUTMEM (da,   stk[0], srcdst);
       PUTMEM (da+2, stk[1], srcdst);
@@ -571,7 +571,7 @@ puttbl (uint16 dr, uint16 dt, uint16 *tbl, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       da &= 0xFFFE;
       PUTMEM (da,   tbl[0], srcdst);
       PUTMEM (da+2, tbl[1], srcdst);
@@ -630,7 +630,7 @@ putquad (uint16 dr, uint16 dt, t_uint64 dval, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEM (da, (uint16)((dval >> 48) & 0xFFFF), srcdst);
       PUTMEM (da+2, (uint16)((dval >> 32) & 0xFFFF), srcdst);
       PUTMEM (da+4, (uint16)((dval >> 16) & 0xFFFF), srcdst);
@@ -2497,7 +2497,7 @@ proc16op (uint16 inst)
 	 lsval <<= ps;
 	 cmplongzero (lsval);
 	 if (sign)
-	    (int32)lsval >>= (16 - wd);
+	    lsval = (int32)lsval >> (16 - wd);
 	 else
 	    lsval >>= (16 - wd);
 	 sval = (uint16)(lsval >> 16);
@@ -2513,9 +2513,9 @@ proc16op (uint16 inst)
 	 sval <<= ps;
 	 cmpwordzero (sval);
 	 if (sign)
-	    (int16)sval >>= 16 - wd;
+	    sval = (int16)sval >> (16 - wd);
 	 else
-	    sval >>= 16 - wd;
+	    sval >>= (16 - wd);
       }
       putword (dr, dt, sval, DST);
 #ifdef DEBUGEINSTR
@@ -2679,7 +2679,7 @@ proc18op (uint16 inst)
       {
 	 uint16 st;
 	 st = statreg & 0x01DF;
-         st |= GETREG(r) & 0xFC20;
+         st |= (GETREG(r) & 0xFC20);
 	 statreg = st;
       }
       else
