@@ -131,7 +131,7 @@ getfloat (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = ((uint32)GETMEM (sa & 0xFFFE, srcdst) << 16) | 
 	      (uint32)GETMEM ((sa+2) & 0xFFFE, srcdst);
       break;
@@ -191,7 +191,7 @@ getdouble (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = ((t_uint64)GETMEM (sa & 0xFFFE, srcdst) << 48) | 
 	     ((t_uint64)GETMEM ((sa+2) & 0xFFFE, srcdst) << 32) |
 	     ((t_uint64)GETMEM ((sa+4) & 0xFFFE, srcdst) << 16) |
@@ -252,7 +252,7 @@ putfloat (uint16 dr, uint16 dt, uint32 dval, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEM (da, (uint16)((dval >> 16) & 0xFFFF), srcdst);
       PUTMEM (da+2, (uint16)(dval & 0xFFFF), srcdst);
       break;
@@ -313,7 +313,7 @@ putdouble (uint16 dr, uint16 dt, t_uint64 dval, int srcdst)
 	    fprintf (tracefd, " @>%X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEM (da, (uint16)((dval >> 48) & 0xFFFF), srcdst);
       PUTMEM (da+2, (uint16)((dval >> 32) & 0xFFFF), srcdst);
       PUTMEM (da+4, (uint16)((dval >> 16) & 0xFFFF), srcdst);
