@@ -58,6 +58,7 @@ extern int traceend;
 extern int tracelim[2];
 extern int deferint;
 extern int traceit;
+extern int windowlen;
 extern uint16 mapcrudata;
 extern uint16 curmapindex;
 extern uint16 errcrudata;
@@ -699,7 +700,7 @@ getaddr (uint16 sr, uint16 st, int inc)
 	    fprintf (tracefd, " @>%04X(%d)", sa, sr);
       }
       pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       break;
 
    case 3:
@@ -754,7 +755,7 @@ getbyte (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%04X(%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = GETMEMB (sa, srcdst);
       break;
 
@@ -809,7 +810,7 @@ getword (uint16 sr, uint16 st, int inc, int srcdst)
 	    fprintf (tracefd, " @>%04X(R%d)", sa, sr);
       }
       if (inc) pcreg += 2;
-      if (sr != 0) (int16)sa += (int16)GETREG (sr);
+      if (sr != 0) sa = (int16)sa + (int16)GETREG (sr);
       sval = GETMEM (sa & 0xFFFE, srcdst);
       break;
 
@@ -877,7 +878,7 @@ putbyte (uint16 dr, uint16 dt, uint8 dval, int srcdst)
 	    fprintf (tracefd, " @>%04X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEMB (da, dval, srcdst);
       break;
 
@@ -930,7 +931,7 @@ putword (uint16 dr, uint16 dt, uint16 dval, int srcdst)
 	    fprintf (tracefd, " @>%04X(%d)", da, dr);
       }
       pcreg += 2;
-      if (dr != 0) (int16)da += (int16)GETREG (dr);
+      if (dr != 0) da = (int16)da + (int16)GETREG (dr);
       PUTMEM (da, dval, srcdst);
       break;
 
@@ -1468,7 +1469,7 @@ proc3op (uint16 inst)
          fprintf (tracefd, " %d", reg);
       op = statreg;
       SET_XOP;
-      CLR_NONPRV; /* Privleged */
+      CLR_NONPRV; /* Privileged */
       CLR_MAP; /* Use Map 0 */
       CLR_OVERI;
       CLR_WCS;
@@ -1586,7 +1587,7 @@ proc5op (uint16 inst)
       if (traceit)
 	 fprintf (tracefd, " val=>%04X", val);
       acc = acc << 16;
-      (int32)acc = (int32)acc >> cnt;
+      acc = (int32)acc >> cnt;
       if (acc & 0x00008000) SET_CARRY;
       else CLR_CARRY;
       val = (acc >> 16) & 0xFFFF;
@@ -2263,14 +2264,14 @@ proc7op (uint16 inst)
       }
       val = wpreg;
       CLR_MAP;
-      if (model < 5)
+      if (model == 4 || model == 9)
 	 wpreg = GETMEM0 (0xFFFC) & 0xFFFE;
       else
 	 wpreg = GETMEM0 (TPCSSTART+0xFFFC) & 0xFFFE;
       PUTREG (13, val);
       PUTREG (14, pcreg);
       PUTREG (15, statreg);
-      if (model < 5)
+      if (model == 4 || model == 9)
 	 pcreg = GETMEM0 (0xFFFE) & 0xFFFE;
       else
 	 pcreg = GETMEM0 (TPCSSTART+0xFFFE) & 0xFFFE;
@@ -2590,9 +2591,13 @@ runinst (uint16 inst)
    type = decodeinst (inst);
 
 #if defined(DEBUGINSTR)
-   screenposition (16,1);
-   clearline ();
+   if (enablepanel)
+   {
+      screenposition (windowlen,1);
+      clearline ();
+   }
    printf ("INST = %04X TYPE = %d", inst, type);
+   if (!enablepanel) printf ("\n");
 #endif
 
    deferint = 0;
